/*  VER 033   TAB P   $Id: bounce.c,v 1.2 1997/08/14 13:39:42 src Exp $
 *
 *  bounce message, i.e. return to sender
 *
 *  copyright 1997 Egil Kvaleberg, egil@kvaleberg.no
 *  the GNU General Public License applies
 */

#include "common.h"
#include "proto.h"
#include "options.h"

static void 
do_bounce(FILE *f,char *to,FILE *art,char *filename,char *msgid,char *reason);

/*
 *  bounce the message
 */
int  
bounce_msg(char *to, FILE *art,char *filename, char *msgid, char *reason)
{
    int n;
    int pid;
    FILE *f;

    progtitle("bounce message");
#if HAVE_SENDMAIL
    /* send bounce if there is a return address */
    if (!to || !to[0]) return 0;
    log_msg(L_DEBUG4,"forking...");
    if (!(f=fork_write(SENDMAIL_PATH,&pid,to,NULL))) {
	log_msg(L_ERR,"can't run sendmail: %s",SENDMAIL_PATH);
	return 0;
    }
    log_msg(L_DEBUG4,"write...");
    /* generate mail message */
    do_bounce(f,to,art,filename,msgid,reason);
    fclose(f);

    /* wait for child to finish */
    log_msg(L_DEBUG4,"wait pid %d...",pid);
    n = wait_pid(pid);
    log_msg(L_DEBUG,"sendmail status is %d",n);
    return n==0;
#else 
    log_msg(L_ERR,"sendmail not available");
    return 0;
#endif
}

/*
 *  generate the bounce message
 */
static void 
do_bounce(FILE *f,char *to, FILE *art,char *filename,char *msgid,char *reason)
{
    char *sep = "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX"; /* BUG: */

    /* BUG: check for EOF on f? */
    fprintf(f,"To: %s\n",to);
    fprintf(f,"Subject: News posting failed\n"); /* BUG: */
    fprintf(f,"Auto-Submitted: auto-generated (failure)\n");
    if (attach == 0) {
	fprintf(f,"MIME-Version: 1.0\n");
	fprintf(f,"Content-Type: MULTIPART/MIXED; BOUNDARY=\"%s\"\n",sep);
    }
    fprintf(f,"\n");

    if (attach == 0) {
	fprintf(f,"--%s\n",sep);
	fprintf(f,"Content-Type: TEXT/PLAIN; charset=US-ASCII\n\n");
    }
    fprintf(f,"   ----- The following news message could not be posted -----\n");
    fprintf(f,"Message-ID: %s\n",msgid);
    fprintf(f,"Spool file: %s\n",filename);
    if (art) {
	fprintf(f,"Entered: %s\n",text_time(what_date(art,filename)));
    }
    fprintf(f,"Host: %s %s\n\n",
	       hostname,strcmp(hostport,NNTP_PORT)==0 ? "" : hostport);
    fprintf(f,"   ----- Reason for failure -----\n");
    fprintf(f,"%s\n\n",reason);

    if (attach != 'n') {
	if (!art) {
	    fprintf(f,"   ----- Message text not available -----\n");
	} else {
	    char line[1024];
	    int n;
    
	    fprintf(f,"   ----- Complete message follows -----\n");
	    if (attach == 0) {
		fprintf(f,"--%s\n",sep);
		fprintf(f,"Content-Type: TEXT/PLAIN; charset=%s\n",
						 /* BUG: */ "US-ASCII");
		fprintf(f,"Content-ID: %s\n",msgid);
		fprintf(f,"Content-Description: Failed posting to %s\n\n",
								    hostname);
	    }
	    /* rewind file */
	    fseek(art,0L,0);
	    /* and copy */
	    while ((n=fread(line, 1, sizeof(line), art)) > 0) {
		/* BUG: no error check */
		fwrite(line, 1, n, f);
	    }
	}
    }
}	     


